import type { CodeGeneratorRequest, DescFile } from "@bufbuild/protobuf";
import { CodeGeneratorResponse } from "@bufbuild/protobuf";
import type { FileInfo, GeneratedFile } from "./generated-file.js";
import { RuntimeImports } from "./runtime-imports.js";
import type { Target } from "./target.js";
import { RewriteImports } from "./import-path.js";
/**
 * Schema describes the files and types that the plugin is requested to
 * generate.
 */
export interface Schema {
    /**
     * The files we are asked to generate.
     */
    readonly files: readonly DescFile[];
    /**
     * All files contained in the code generator request.
     */
    readonly allFiles: readonly DescFile[];
    /**
     * The plugin option `target`. A code generator should support all targets.
     */
    readonly targets: readonly Target[];
    /**
     * Provides some symbols from the runtime library @bufbuild/protobuf.
     */
    readonly runtime: RuntimeImports;
    /**
     * Generate a new file with the given name.
     */
    generateFile(name: string): GeneratedFile;
    /**
     * The original google.protobuf.compiler.CodeGeneratorRequest.
     */
    readonly proto: CodeGeneratorRequest;
}
interface SchemaController {
    schema: Schema;
    getFileInfo: () => FileInfo[];
}
export declare function createSchema(request: CodeGeneratorRequest, targets: Target[], tsNocheck: boolean, bootstrapWkt: boolean, rewriteImports: RewriteImports, importExtension: string, keepEmptyFiles: boolean, pluginName: string, pluginVersion: string, pluginParameter: string): SchemaController;
export declare function toResponse(files: FileInfo[]): CodeGeneratorResponse;
export {};
